/*******************************************************************************
 * Copyright (c) 2013, SWITCH
 ******************************************************************************/
package ch.SWITCH.uniapp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;
import android.widget.Toast;

public class AlternateLoginActivity extends BaseLoginActivity {
	private static final String LOG_TAG = "AlternateLoginActivity";

	/** request code for scanning a QR code */
	private static final int REQUEST_CODE_SCAN_QR = 1;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_aailogin_alt);

		// button to share the login URL
		ImageButton vbShare = (ImageButton) findViewById(R.id.url_share);
		vbShare.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent sendIntent = new Intent(Intent.ACTION_SEND);
				sendIntent.setType("text/plain");
				sendIntent.putExtra(Intent.EXTRA_SUBJECT, "Login for Uniapp");
				sendIntent.putExtra(Intent.EXTRA_TEXT, "Please visit " + getString(R.string.uniapp_url_alt_login));
				startActivity(sendIntent);
			}
		});

		// button scan a QR code
		Button vbQr = (Button) findViewById(R.id.button_qr);
		vbQr.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent intent = new Intent("com.google.zxing.client.android.SCAN");
				if (getPackageManager().resolveActivity(intent, 0) == null) {
					Log.d(LOG_TAG, "User has zXing barcode scanner not installed");
					showMarketBarcodeScanner();
				} else {
					intent.putExtra("SCAN_MODE", "QR_CODE_MODE");
					startActivityForResult(intent, REQUEST_CODE_SCAN_QR);
				}
			}
		});

		// button send the entered token to the server
		final Button vbEnter = (Button) findViewById(R.id.button_alt_enter);
		final EditText vtToken = (EditText) findViewById(R.id.input_token);
		vtToken.setOnEditorActionListener(new OnEditorActionListener() {

			@Override
			public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
				if (event.getAction() != KeyEvent.ACTION_DOWN)
					return false; // we only want to trigger it on keydown

				new GetAccessTokenTask(vtToken.getText()
						.toString()).execute();

				return true;
			}
		});
		vbEnter.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				new GetAccessTokenTask(vtToken.getText()
						.toString()).execute();
			}
		});

		// listener to enable the enter button only when there is enough text in
		// it
		// FIXME doesn't seem to work
		vtToken.addTextChangedListener(new TextWatcher() {

			@Override
			public void onTextChanged(CharSequence s, int start, int before, int count) {
				if (s.length() > 4) {
					vbEnter.setEnabled(true);
				}

			}

			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}

			@Override
			public void afterTextChanged(Editable s) {
			}
		});
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == REQUEST_CODE_SCAN_QR) {
			if (resultCode == RESULT_CANCELED) {
				Log.d(LOG_TAG, "User canceled QR scan");
				return;
			}

			// handle the QR code scan
			String authCode = data.getStringExtra("SCAN_RESULT");
			if (authCode == null) {
				showErrorDialog("Could not get data from QR code", false);
				Log.e(LOG_TAG, "QR scan yielded no String!");
			} else {
				new GetAccessTokenTask(authCode).execute();
			}
		} else {
			Log.w(LOG_TAG, "Received activity result with unknown requestCode");
		}
	}

	/**
	 * Displays an alert dialog that allows the user to download the zxing QR
	 * scanner on gPlay
	 */
	protected void showMarketBarcodeScanner() {
		AlertDialog alertDialog = new AlertDialog.Builder(this).create();
		;
		alertDialog.setMessage("Required QR reader is not installed. Click Ok to go to the market page now");
		alertDialog.setButton("Ok", new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				Intent intent = new Intent(Intent.ACTION_VIEW);
				intent.setData(Uri.parse("market://details?id=com.google.zxing.client.android"));
				startActivity(intent);
				dialog.dismiss();

			}
		});

		alertDialog.setButton2("Cancel", new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();

			}
		});
		alertDialog.show();

	}

	/**
	 * @author latzer
	 * 
	 *         AsyncTask to swap an auth code for an access token
	 */
	private class GetAccessTokenTask extends AsyncTask<Void, Void, JSend> {
		private String authCode;
		private ProgressDialog pd;

		public GetAccessTokenTask(String authCode) {
			this.authCode = authCode;
		}

		protected String getStringResponse(HttpResponse response) throws IOException {
			BufferedReader rd = new BufferedReader(new InputStreamReader(response.getEntity()
					.getContent()));

			StringBuffer bResponse = new StringBuffer();
			String line;
			while ((line = rd.readLine()) != null) {
				bResponse.append(line);
			}
			return bResponse.toString();

		}

		private JSend buildErrorResponse(JSONObject error) throws JSONException {
			Log.e(LOG_TAG, error.getString("error"));
			JSend out = new JSend();
			out.setStatus(JSend.ERROR);
			int code;
			if (error.has("error_code")) {
				code = error.getInt("error_code");
			} else {
				code = -1;
			}
			if (code == StatusCode.ERROR_INVALID_GRANT) {
				out.setMessage("Code is invalid");
			} else {
				out.setMessage("An error occured while logging you in.");
				// should it be more specific?
			}
			return out;

		}

		@Override
		protected JSend doInBackground(Void... nothing) {
			Uri uri = Util.getApiUriBuilder(AlternateLoginActivity.this)
					.appendPath("access_token")
					.build();
			JSend jsend = new JSend();
			HttpClient client = new DefaultHttpClient();
			HttpPost post = new HttpPost(uri.toString());
			List<NameValuePair> params = new ArrayList<NameValuePair>();
			params.add(new BasicNameValuePair("grant_type", "authorization_code_manual"));
			params.add(new BasicNameValuePair("client_id", getString(R.string.uniapp_client_id)));
			params.add(new BasicNameValuePair("code", authCode));

			Log.d(LOG_TAG, "Requesting " + uri.toString());
			try {
				post.setEntity(new UrlEncodedFormEntity(params));

				HttpResponse httpresponse = client.execute(post);
				Log.v(LOG_TAG, "HTTP Status " + httpresponse.getStatusLine()
						.getStatusCode());

				String response = getStringResponse(httpresponse);
				Log.d(LOG_TAG, "response: " + response);

				JSONObject json = new JSONObject(response);

				if (json.has("error")) {
					jsend = buildErrorResponse(json);

				} else { // set the access token from the response
					String token = json.getString("access_token");
					int expires = json.getInt("expires");
					setAccessToken(getApplicationContext(), token, expires);
					jsend.setStatus(JSend.SUCCESS);
				}
			} catch (JSONException e) {
				String message = "Error parsing response";
				Log.e(LOG_TAG, message, e);
				jsend.setStatus(JSend.ERROR)
				.setMessage(message);
			} catch (Throwable e) {
				String message = "Error communicating with server";
				Log.e(LOG_TAG, message, e);
				jsend.setStatus(JSend.ERROR)
				.setMessage(message);
			}

			return jsend;
		}

		@Override
		protected void onPostExecute(JSend result) {
			pd.cancel();
			if (JSend.SUCCESS.equals(result.getStatus())) {
				try {
					/*
					 * FIXME when hitting the back key after successfully
					 * logging in, the user will land in the AAILoginActivity
					 * which is in the backstack before this activity - this
					 * activity is finished, but i didn't found a way to throw
					 * AAILoginActivity out of the back stack.
					 */
					Context ctx = getApplicationContext();
					new QueryAndLauchHomeTask(new AAIAttributeStorage(ctx), getAccessToken(ctx)).execute();
				} catch (TokenExpiredException e) {
					Log.e(LOG_TAG, "Token expired right after acquiring it"); // ...which
					// obviously shouldn't happen
					Toast.makeText(getApplicationContext(), "Login is invalid", Toast.LENGTH_LONG)
					.show();
				}
			} else {
				handleError(result);
			}
			super.onPostExecute(result);
		}

		@Override
		protected void onPreExecute() {
			pd = ProgressDialog.show(AlternateLoginActivity.this, "", "Validating code", true);
			super.onPreExecute();
		}

	}

}
