/*******************************************************************************
 * Copyright (c) 2013, SWITCH
 ******************************************************************************/
package ch.SWITCH.uniapp;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.net.Uri;
import android.widget.Toast;

/**
 * @author latzer
 * 
 *         AsyncTask to delete the current access token locally and remotely. If
 *         the remote logout fails, the user is not logged out locally. On
 *         success, the current activity is finished and the login activity is
 *         launched.
 */
public class LogoutTask extends ApiCallTask {
	protected Activity ctx;
	private ProgressDialog progressDialog;
	private static final String LOG_TAG = "LogoutTask";

	public LogoutTask(Activity ctx) {
		this.ctx = ctx;
	}

	@Override
	protected JSend doInBackground(String... params) {
		String accessToken;
		try {
			accessToken = AAILoginActivity.getAccessToken(ctx);
			Uri uri = Util.getApiUriBuilder(ctx)
					.appendPath("revoke")
					.appendQueryParameter("access_token", accessToken)
					.build();
			return apiCallJSend(uri.toString());
		} catch (TokenExpiredException e) {
			// that's fine, we wanted to invalidate it anyway
			return new JSend().setStatus(JSend.SUCCESS);
		}
	}

	@Override
	protected void onPostExecute(JSend result) {
		progressDialog.dismiss();
		if (result != null && result.getStatus()
				.equals(JSend.SUCCESS)) {
			Toast.makeText(ctx, "Logout successful", Toast.LENGTH_SHORT)
			.show();

			AAILoginActivity.setAccessToken(ctx, null, 0);

			// back to login activity
			Intent intent = new Intent(ctx, AAILoginActivity.class);
			intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
			ctx.startActivity(intent);
			ctx.finish();
		} else {
			Toast.makeText(ctx, "Unable to log you out at this time, please try again later", Toast.LENGTH_LONG)
			.show();
		}
		super.onPostExecute(result);
	}

	@Override
	protected void onPreExecute() {
		progressDialog = ProgressDialog.show(ctx, "", "Logging out...", true);
		super.onPreExecute();
	}

}
