/*******************************************************************************
 * Copyright (c) 2013, SWITCH
 ******************************************************************************/
package ch.SWITCH.uniapp.demo;

import java.util.ArrayList;
import java.util.Map;

import android.app.Activity;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.ViewGroup;
import android.widget.TableLayout;
import android.widget.TextView;
import android.widget.Toast;
import ch.SWITCH.uniapp.AAILoginActivity;
import ch.SWITCH.uniapp.AAIAttributeStorage;
import ch.SWITCH.uniapp.ApiCallTask;
import ch.SWITCH.uniapp.JSend;
import ch.SWITCH.uniapp.LogoutTask;
import ch.SWITCH.uniapp.R;
import ch.SWITCH.uniapp.TokenExpiredException;

import com.google.gson.internal.StringMap;

/**
 * @author latzer
 * 
 *         Demo activity displaying the AAI attributes retrieved from the Mobile
 *         Proxy and a Timetable from a resource server, using the login from
 *         the Mobile Proxy
 * 
 */
public class HomeActivity extends Activity {

	public String LOG_TAG = "HomeActivity";

	/**
	 * Truncates the given String to 20 characters
	 */
	private String truncateAttributeName(String name) {
		if (name.length() <= 20)
			return name;
		return name.substring(0, 5) + "..." + name.substring(name.length() - 12, name.length());
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_home);

		// write the AAI attributes to a table
		TableLayout vgAttributes = (TableLayout) findViewById(R.id.table_attributes);
		Map<String, ?> attributes = new AAIAttributeStorage(this).getAll();

		for (String key : attributes.keySet()) {
			ViewGroup vRow = (ViewGroup) LayoutInflater.from(this)
					.inflate(R.layout.keyvaluerow, vgAttributes, false);
			TextView vKey = (TextView) vRow.findViewById(R.id.key);
			TextView vValue = (TextView) vRow.findViewById(R.id.value);

			/*
			 * quick & dirty hack: truncating the attribute name so super long
			 * attributes don't blow up the layout ...Android GUIs are hard.
			 */
			vKey.setText(truncateAttributeName(key));
			vValue.setText((String) attributes.get(key));

			vgAttributes.addView(vRow);
		}

		// retrieve and populate the timetable
		TableLayout vgTimetable = (TableLayout) findViewById(R.id.table_time);
		new GetTimetableTask(vgTimetable).execute();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_home, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_logout:
			new LogoutTask(this).execute();
			break;
		}
		return super.onOptionsItemSelected(item);
	}

	/**
	 * @author latzer
	 * 
	 *         AsyncTask for retrieving a timetable from a Mobile Proxy enabled
	 *         resource server
	 */
	private class GetTimetableTask extends ApiCallTask {
		protected TableLayout timetable;

		protected GetTimetableTask(TableLayout timetable) {
			this.timetable = timetable;
		}

		@Override
		protected JSend doInBackground(String... params) {
			Log.d(LOG_TAG, "requesting timetable");
			Uri requestUri;
			try {
				requestUri = new Uri.Builder().scheme("https")
						.authority("edulab.switch.ch")
						.path("mobile-proxy-resource/api.php")
						.appendQueryParameter("access_token",
								AAILoginActivity.getAccessToken(HomeActivity.this))
								.build();
				return apiCallJSend(requestUri.toString());
			} catch (TokenExpiredException e) {
				return new JSend().setStatus(JSend.FAIL)
						.setMessage("Access Token expired");
			}
		}

		@Override
		protected void onPostExecute(JSend result) {
			if (result.getStatus()
					.equals(JSend.SUCCESS)) {
				// read the timetable, populate the view
				ArrayList<StringMap> data = (ArrayList) result.getData();
				for (StringMap row : data) {
					ViewGroup vRow = (ViewGroup) LayoutInflater.from(getApplicationContext())
							.inflate(R.layout.keyvaluerow, timetable, false);
					TextView vTime = (TextView) vRow.findViewById(R.id.key);
					TextView vDescription = (TextView) vRow.findViewById(R.id.value);

					vTime.setText(row.get("time")
							.toString());
					vDescription.setText(row.get("description")
							.toString());

					timetable.addView(vRow);
				}
			} else {
				Log.e(LOG_TAG, "error retrieving timetable: " + result.getMessage());
				Toast.makeText(getApplicationContext(), "Error retrieving timetable", Toast.LENGTH_SHORT)
				.show();
			}
			super.onPostExecute(result);
		}
	}
}
