# Mobile Proxy API

This code is based on https://github.com/lncd/oauth2-example-auth-server

## Installation
### Requirements
PHP 5.3
MySQL Database

### General
* Project folder needs to be accessible in a web server
* `templates/cache` and `templates/vendor` need to be writable by the web server
* config.php.dist is used to create a config.php. Make sure the logfile is writable

### Setting up libraries
Composer is used to manage external libraries. A quick guide to install it locally into the project folder is found here: http://getcomposer.org/doc/00-intro.md#installation-nix
Then one can simply run `php composer.phar install` in the installation directory to install the dependencies.
`php composer.phar update` can later be used to update dependencies, should the need arise.

### Setting up database
Database can be set up using the provided DATABASE.sql. The database access parameters in config.php need to be adjusted accordingly.

### Apache Config
It is important to protect the following paths with shibboleth:
	<Directory "/opt/mobile-proxy-*">
		Allow from all
	</Directory>

	Alias /mobile-proxy-auth /opt/mobile-proxy-auth/

	<LocationMatch "/mobile-proxy-auth/aailogin/.*">
					AuthType shibboleth
					ShibRequireSession On
					require valid-user
	</LocationMatch>
	<LocationMatch "/mobile-proxy-auth/admin.php.*">
					AuthType shibboleth
					ShibRequireSession On
					require valid-user
	</LocationMatch>

The following snippet can help with apache rewrites to make access to the REST api briefer. If this is used, the API_URL in config.php needs to be updated.

	RewriteEngine On
	RewriteCond %{REQUEST_URI} !^/mobile-proxy-auth/admin.php.*
	RewriteCond %{REQUEST_URI} !^/mobile-proxy-auth/css.*
	RewriteCond %{REQUEST_URI} !^/mobile-proxy-auth/resources.*
	RewriteCond %{REQUEST_URI} !^/mobile-proxy-auth/lib/qr.*
	RewriteRule ^/mobile-proxy-auth(/.*) /mobile-proxy-auth/oauth.php$1 [PT]


This snippet makes the alternative login URL as short as possible
	RedirectMatch ^/alt/(.*) https://edulab.switch.ch/mobile-proxy-auth/alt/$1

## Setting up an app
The following steps need to be made to register a new app that can authenticate with the mobile proxy
### Create client ID and store it in database
Since no administration GUI exists, the database needs to be modified directly. Two entries need to be made. 

New entry in the table `oauth_clients`
* `id` : an alphanumerical ~20byte string
* `secret` : an alphanumerical ~20byte string (currently unused for apps)
* `name` : a unique identifier for your app, used to distinguish the app in API calls (this includes the alternative login URL)
* `prettyName` : the app name to be displayed in the token administration interface

New entry in the table `oauth_client_endpoints`
* `id` : unique integer ID
* `client_id` : the `id` just created for `oauth_clients`
* `redirect_uri` : usually "uniapp://[`id`]"

### Configure application
The strings `uniapp_id`, `uniapp_client_id` and `uniapp_url_alt_login` need to be adjusted to the generated values.

## Setting up a resource server
The following steps need to be made to register a new resource server that can authenticate with the mobile proxy
### Create client ID and store it in database
An entry in `oauth_clients` needs to be created in the same way as for the app above. It is important to know that the field `secret` is used here. No entry needs to be made in `oauth_client_endpoints`.

## Routes
These are all the routes the API provides, and the parameters required to access them.
### `oauth.php`
| Method   | Route                   | Required Parameters                               | Description                                                                                                        |
| -------- | ----------------------- | ------------------------------------------------- | ------------------------------------------------------------------------------------------------------------------ |
| GET      | `/signin/:app`          | client_id, redirect_uri, response_type, scope     | validates OAuth2 parameters, redirects to `/aailogin/:app`                                                         |
| GET      | `/aailogin/:app`        | -                                                 | shibboleth protected, redirects to the `redirect_uri` into the app after the AAI login                             |
| GET      | `/alt/:app`             | -                                                 | initiates an alternate (desktop browser) login that yields an authcode that can be manually entered into the app   |
| POST     | `/access_token`         | client_id, client_secret, code                    | exchanges the code for an access token                                                                             |
| GET      | `/devicename`           | -                                                 | Shows a HTML form querying the User for a device name to be associated with the login (used for alternate login)   |
| POST     | `/devicename`           | device                                            | Stores given device name and proceeds to `/display_code`                                                           |
| GET      | `/display_code/:code`   | -                                                 | Displays :code on a HTML page, prompting the user to enter it into their device                                    |
| GET      | `/validate`             | access_token + basic auth header client_id:secret | validates given `access_token`, returning the `app_id` the token belongs to                                        |
| GET      | `/attributes`           | access_token                                      | Returns the released attributes as JSON.                                                                           |

## Admin page
`admin.php` needs to be secured with AAI. When authenticated, all valid access tokens issued to the user are displayed and can be revoked.
