<?php
/* Copyright (c) 2013, SWITCH */

use OAuth2\Exception;
use OAuth2\AuthServer;
use OAuth2\Util\SecureKey;

/**
 * This grant type is meant to be initiated from a desktop browser to complete
 * AAI logins which are hard or impossible to do on a phone due to technical
 * reasons (e.g. certificate login). It produces a short Authorization Token
 * meant to be input into the phone manually or read via QR code.
 */
class AuthCodeManual extends OAuth2\Grant\AuthCode{

	protected $identifier = 'authorization_code_manual';

	protected $responseType = 'code_manual';

	public function completeFlow($inputParams = null) {

		// Get the required params
		$authParams = AuthServer::getParam(array('client_id', 'client_secret', 'code'), 'post', $inputParams);

		if (is_null($authParams['client_id'])) {
			throw new Exception\ClientException(sprintf(AuthServer::getExceptionMessage('invalid_request'), 'client_id'), 0);
		}

		// Validate client ID 
		$clientDetails = AuthServer::getStorage('client')->getClient($authParams['client_id'], $authParams['client_secret'], 'none');

		if ($clientDetails === false) {
			throw new Exception\ClientException(AuthServer::getExceptionMessage('invalid_client'), 8);
		}

		$authParams['client_details'] = $clientDetails;

		// Validate the authorization code
		if (is_null($authParams['code'])) {
			throw new Exception\ClientException(sprintf(AuthServer::getExceptionMessage('invalid_request'), 'code'), 0);
		}

		// Verify the authorization code matches the client_id and the request_uri
		$session = AuthServer::getStorage('session')->validateAuthCode($authParams['client_id'], 'none', $authParams['code']);

		if ( ! $session) {
			throw new Exception\ClientException(sprintf(AuthServer::getExceptionMessage('invalid_grant'), 'code'), 9);
		}

		// A session ID was returned so update it with an access token,
		//	remove the authorisation code, change the stage to 'granted'

		$accessToken = SecureKey::make();
		$refreshToken = (AuthServer::hasGrantType('refresh_token')) ? SecureKey::make() : null;

		$accessTokenExpires = time() + AuthServer::getExpiresIn();
		$accessTokenExpiresIn = AuthServer::getExpiresIn();

		AuthServer::getStorage('session')->updateSession(
			$session['id'],
			null,
			$accessToken,
			$refreshToken,
			$accessTokenExpires,
			'granted'
		);

		$response = array(
			'access_token'	=>	$accessToken,
			'token_type'	=>	'bearer',
			'expires'		=>	$accessTokenExpires,
			'expires_in'	=>	$accessTokenExpiresIn
		);

		if (AuthServer::hasGrantType('refresh_token')) {
			$response['refresh_token'] = $refreshToken;
		}

		return $response;
	}
}
